/***************************************************************************

    analyzer.C

    Copyright (C) 1996 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: analyzer.C,v 0.27 1996/02/17 06:59:39 drewvel Exp $ */

/*******************************************************************/
#include <iostream.h>
#include <iomanip.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "analyzer.h"
#include "common.h"
#include "defvals.h"
#include "gconfig.h"


/*******************************************************************/
TAnalyzer::TAnalyzer(int& argc, char *argv[], const globalConfig& gConfig)
  :gconfig_default(gConfig)
{
  DEBUGNLF("TAnalyzer constructing -- ");
  
  ProgramInfo();

  gconfig=gConfig;

  if (!ProcessArgs(argc, argv, gconfig)) 
    exit(1);
  
  DEBUG("done");
}


/*******************************************************************/
void 
TAnalyzer::ProgramInfo()
{
  extern char* gProgInfo;
  cout << endl << gProgInfo << endl;
}


/*******************************************************************/
void 
TAnalyzer::Usage()
{
  cerr << "\nUsage:  " NAME " [options]" << endl
    << endl << "See man page for more info." << endl << endl;
  cerr << endl;
}


/*******************************************************************/
static void
ParseColor(ColorOption& co, const char *argv)
{
  sscanf(argv, "rgb:%2x/%2x/%2x", &co.r, &co.g, &co.b);
  DEBUG(argv << " is RGB of " << co.r << " " << co.g << " " << co.b);
}


/*******************************************************************/
bool 
TAnalyzer::ProcessArgs(int argc, char *args[], globalConfig& config)
#define CheckRange(a) { if ((a)>=argc) { Usage(); exit(1); } }
{
  int i, j;
  bool knownarg;

  i=1;
  while (i<argc) {
    DEBUG("arg: " << args[i] << ": ");
    knownarg=false;
    
    StrOption* p=(StrOption *) &config;
    for (j=0; j<NUMSTROPTIONS && !knownarg; j++) {
      if (strcmp(args[i], p->name)==0) {
	i++;
	CheckRange(i);
	strcpy(p->val, args[i]);
	knownarg=true;
      }
      p++;
    }
    
    IntOption* q=(IntOption *) p;
    for (j=0; j<NUMINTOPTIONS && !knownarg; j++) {
      if (strcmp(args[i], q->name)==0) {
	i++;
	CheckRange(i);
	q->val=atoi(args[i]);
	knownarg=true;
      }
      q++;
    }

    RealOption* r=(RealOption *) q;
    for (j=0; j<NUMREALOPTIONS && !knownarg; j++) {
      if (strcmp(args[i], r->name)==0) {
	i++;
	CheckRange(i);
	r->val=atof(args[i]);
	knownarg=true;
      }
      r++;
    }

    ColorOption* c=(ColorOption *) r;
    for (j=0; j<NUMCOLOROPTIONS && !knownarg; j++) {
      if (strcmp(args[i], c->name)==0) {
	i++;
	CheckRange(i);
	ParseColor(*c, args[i]);
	knownarg=true;
      }      
      c++;
    }

    BoolOption* b=(BoolOption *) c;
    for (j=0; j<NUMBOOLOPTIONS && !knownarg; j++) {

      // check true sense
      if (strcmp(args[i], b->name)==0) {
	b->val=true;
	knownarg=true;
      }
      
      // check false sense
      char falsename[64];
      strcpy(falsename, "-no");
      strcat(falsename, b->name+1);
      if (strcmp(args[i], falsename)==0) {
	b->val=false;
	knownarg=true;
      }

      b++;
    }    

    if (!knownarg) { 
      cerr << "error: unknown parameter " << args[i] << endl;
      return false;
    }

    i++;
  }  

  if (config.HighFreq.val<0)
    config.HighFreq.val=config.SampleRate.val/2;

  return true;
}


/*******************************************************************/
// this function performs a "diff" on the config and outputs 
// appropriate cmd-line argument info for a "patch"
void
TAnalyzer::gConfigDiff(const globalConfig& gc_orig, const globalConfig& gc_new,
		       ostream& out) const
{
  int i;

  StrOption *p=(StrOption *) &gc_new, *p1=(StrOption *) &gc_orig;
  for (i=0; i<NUMSTROPTIONS; i++, p++, p1++) {
    if (strcmp(p1->val, p->val)!=0)
      out << p->name << " " << p->val << " ";
  }
  
  IntOption *q=(IntOption *) p, *q1=(IntOption *) p1;
  for (i=0; i<NUMINTOPTIONS; i++, q++, q1++) {
    if (q->val!=q1->val)
      out << q->name << " " << q->val << " ";
  }

  RealOption *r=(RealOption *) q, *r1=(RealOption *) q1;
  for (i=0; i<NUMREALOPTIONS; i++, r++, r1++) {
    if (r->val!=r1->val) {
      char tmp[64];
      sprintf(tmp, "%f", r->val);
      out << r->name << " " << tmp << " ";
    }
  }

  ColorOption *c=(ColorOption *) r, *c1=(ColorOption *) r1;
  for (i=0; i<NUMCOLOROPTIONS; i++, c++, c1++) {
    if (*c!=*c1) {
      out << c->name << " " 
	  << "rgb:" << hex << setw(2)
	  << c->r << "/" << c->g << "/" << c->b << " ";
    }
  }  

  BoolOption *b=(BoolOption *) c, *b1=(BoolOption *) c1;
  for (i=0; i<NUMBOOLOPTIONS; i++, b++, b1++) {
    if (b->val != b1->val)
      if (b->val) out << b->name << " ";
      else out << "-no" << (b->name+1) << " ";
  }

  out << endl;
}


/*******************************************************************/
