/**************************************************************************

    buffer.h
  
    Copyright (C) 1995-96 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: buffer.h,v 0.4 1996/02/17 06:54:58 drewvel Exp $ */

#ifndef __BUFFER_H
#define __BUFFER_H
/*******************************************************************/
#include <assert.h>
#include <string.h>

template <class T>
class TBuffer {
private:
  int len, sz;
  T* array;

  void alloc() { array=new T[len]; sz=len*sizeof(T); }
  void clear() { if (array) { delete [] array; array=NULL; } }

public:
  TBuffer() { array=NULL; len=sz=0; }
  TBuffer(int l) { assert(len>0); len=l; alloc(); }
  ~TBuffer() { clear(); }

  int size() const { return sz; }
  int length() const { return len; }
  int length(int l) { int old=len; 
		      len=l; assert(len>0); 
		      clear(); alloc(); 
		      return old; }

  T& operator [] (int i) { assert(i>=0 && i<len); return array[i]; }
  operator T* () { assert(array); return array; }

  TBuffer<T>& operator = (const T& C) 
  { for (int i=0; i<length(); i++) array[i]=C; return *this; }
};


/*******************************************************************/
// type conversion function
template <class T, class U>
inline void
Type_Convert_Buffer(TBuffer<T>& fbuf, TBuffer<U>& tbuf, const T& ofs)
{  
  for (register int i=0; i<fbuf.length(); i++) tbuf[i]=(U) (fbuf[i]+ofs);
}


/*******************************************************************/
#endif /* __BUFFER_H */
