/***************************************************************************

    color.h                                                               

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: color.h,v 1.2 1996/02/08 05:57:56 drewvel Exp $ */

#ifndef __COLOR_H
#define __COLOR_H

/*******************************************************************/
#include <assert.h>

struct ColorOption;

struct TColor {
  int r, g, b;
  
  TColor() { r=g=b=0; }
  TColor(int R, int G, int B) : r(R), g(G), b(B) { }
  
  TColor& operator = (const TColor& o) 
    { r=o.r; g=o.g; b=o.b; return *this; }
  TColor& operator = (const ColorOption&);

  TColor operator + (const TColor& o) const
    { return TColor(r+o.r, g+o.g, b+o.b); }
  TColor operator + (int o) const
    { return TColor(r+o, g+o, b+o); }

  TColor operator - (const TColor& o) const
    { return TColor(r-o.r, g-o.g, b-o.b); }
  TColor operator - (int o) const
    { return TColor(r-o, g-o, b-o); }

  TColor operator * (const TColor& o) const
    { return TColor(r*o.r, g*o.g, b*o.b); }
  TColor operator * (int o) const
    { return TColor(r*o, g*o, b*o); }

  TColor operator / (const TColor& o) const
    {
      assert(o.r!=0 && o.g!=0 && o.b!=0);
      return TColor((int) ((float) r/(float) o.r), 
		    (int) ((float) g/(float) o.g), 
		    (int) ((float) b/(float) o.b));
    }
  TColor operator / (int o) const
    {
      assert(o!=0);
      float d=(float) o;
      return TColor((int) ((float) r / d), 
		   (int) ((float) g / d), 
		   (int) ((float) b / d));
    }
};


/*******************************************************************/
#endif /* __COLOR_H */
