/**************************************************************************

    controls.h                            

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

// $Id: controls.h,v 0.6 1996/02/08 05:57:56 drewvel Exp $


#ifndef __CONTROLS_H
#define __CONTROLS_H

#include <string>

/*******************************************************************/
class AControl {

private:
  string n;

public:  

  AControl():n("<undefined control>") { }

  const string& name () { return n; }
  string name (const string& N) { string old=n; n=N; return old; }
};


/*******************************************************************/
struct Stepper {
  int min;
  int max;
  int step;
  
  Stepper():step(1) { min=max=-1; }

  int StepValue(int v) 
    {
      int retval;
      retval=v+v%step;
      if (min!=max) {
	if (retval>max) return max;
	if (retval<min) return min;
      } else if (min!=-1) 
	if (retval<=min) return min;

      return retval;
    }
  
  int StepUp(int v) { return StepValue(v+step); }
  int StepDown(int v) { return StepValue(v-step); }
};


/*******************************************************************/
struct IntControl:AControl, Stepper {
  int val;

  IntControl():val(step) { }
  
  IntControl& operator = (const int o) { val=StepValue(o); return *this; }
  IntControl& operator ++ () { val=StepUp(val); return *this; }
  IntControl& operator -- () { val=StepDown(val); return *this; }
};


/*******************************************************************/
struct RangeControl:AControl, Stepper {
  int low;
  int high;
  
  int Diff() { return high-low; }
  void Low(int val) { low=StepValue(val); Normalize(); }
  void High(int val) { high=StepValue(val); Normalize(); }

  void Normalize() 
    {
      if (low>high) 
	StepDown(low);
      
      if (low>high) 
	StepUp(high);
    }

  void LowInc() 
    { 
      low=StepUp(low);
      Normalize();
    }

  void LowDec() 
    { 
      low=StepDown(low);
      Normalize();
    }

  void HighInc() 
    { 
      high=StepUp(high);
      Normalize();
    }

  void HighDec() 
    { 
      high=StepDown(high);
      Normalize();
    }
};


/*******************************************************************/
struct BoolControl:public AControl {
  bool isOn;
 
  bool Toggle() { isOn=!isOn; return isOn; }  
  void Set() { isOn=true; }
  void Clear() { isOn=false; }
};



/*******************************************************************/
#endif /* __CONTROLS_H */
