/***************************************************************************

    svga_display.C

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

// $Id: svga_analyzer.C,v 0.27 1996/02/17 20:44:41 drewvel Exp $             


/*******************************************************************/
#include <fstream.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "svga_analyzer.h"
#include "svga_display.h"

#include "../rms.h"
#include "../fftclass.h"
#include "../gconfig.h"
#include "../keymap.h"
#include "../defvals.h"

extern globalConfig gStore;
extern char *gProgInfo,*gAnalyzerHelpText;


/*******************************************************************/
void
TSVGA_Analyzer::init()
{
  display=NULL;
  displayType=0;
  windowSize=0;
}


/*******************************************************************/
TSVGA_Analyzer::TSVGA_Analyzer(globalConfig& gc, int& argc, char *argv[])
:TAnalyzer(argc, argv, gc)
{ 
  DEBUG("TSVGA_Analyzer constructed");
  init();
}


/*******************************************************************/
TSVGA_Analyzer::~TSVGA_Analyzer() 
{ 
  DEBUG("TSVGA_Analyzer destructed");
}


/*******************************************************************/
void 
TSVGA_Analyzer::DeleteDisplay()
{
  DEBUG("delete display()");
  if (display) {
    OnRetrieveConfig(gconfig);
    delete display;
    display=NULL;
    DEBUG("display destroyed");
  }
}


/*******************************************************************/
void
TSVGA_Analyzer::OnUpdateSignalParameters()
{
  DEBUG("TSVGA_Analyzer::OnUpdateSignalParameters");
  if (display->dc.bits8.isOn)
    signal.bits(8);
  else
    signal.bits(16);
  
  signal.sample_rate(display->dc.rate.val);
  display->dc.rate=signal.sample_rate();
  if ((real) display->dc.FreqRange.high>(display->dc.rate.val/2.0))
    display->dc.FreqRange.high=(int) (display->dc.rate.val/2.0);
  signal.channels(1);
}

/*******************************************************************/
int
TSVGA_Analyzer::CurrentMethod(int type)
{
  if (type>0) {
    int old=displayType;
    NewDisplay(type);
    return old;
  } 
  else 
    return displayType;
}


/*******************************************************************/
void 
TSVGA_Analyzer::AutoCalibrate()
{
  char ans=' ';
  while (ans!='y') {
    cerr << endl;
    cerr << (display->dc.bits8.isOn ? " 8" : "16") << "-bit ";
    cerr << "DC Level Adjustment" << endl;
    cerr << "--------------------------" << endl;
    cerr 
      << "This procedure will attempt to automatically determine a suitable" << endl
      << "DC offset to be applied to the incoming signal.  It is a good idea" << endl
      << "to perform this operation in order to obtain a cleaner signal." << endl;
    
    cerr << endl
	 << "Calibration requires that the input level be set to a minimum." << endl;
    cerr << "The value obtained can then be specified on the command-line" << endl
	 << "with the options -ofs8 and -ofs16." << endl 
	 << endl;
    cerr << endl << "Hit enter to perform calibration trial." << endl;
    getchar();

    int ofstmp=0, ofssum=0;
    if (display->dc.bits8.isOn) {
      cerr << "(8-bit mode) Old value: " 
	<< display->dc.ofs8 << endl;
      cerr << "New value: (";
      display->Start();
      for (int k=0; k<TSignal::numcalsamples; k++) {
	ofstmp=signal.calibrate8();
	cerr << ofstmp << (k==(TSignal::numcalsamples-1)?")":"+");
	ofssum+=ofstmp;
      }
      ofstmp=-ofssum/TSignal::numcalsamples;
      display->Stop();
 
      cerr << "\n\n\t\tnew 8-bit offset: " << ofstmp 
	   << "\n\n\t--\taccept? (y=yes/r=redo/c=cancel) ";
      ans=getchar();
      
      if (ans=='y') {
	cerr << "Value changed." << endl;
	display->dc.ofs8=ofstmp;
      } else 
	cerr << "Aborted." << endl;
      
    } else {
      
      cerr << "(16-bit mode) Old value: " 
	   << display->dc.ofs16 << endl;
      cerr << "New value: (";

      display->Start();
      for (int k=0; k<TSignal::numcalsamples; k++) {
	ofstmp=signal.calibrate16();
	cerr << ofstmp << (k==(TSignal::numcalsamples-1)?")":"+");
	ofssum+=ofstmp;
      }
      ofstmp=-ofssum/TSignal::numcalsamples;
      display->Stop();
      
      cerr << "\n\n\t\tnew 16-bit offset: " << ofstmp 
	   << "\n\n\t--\taccept? (y=yes/r=redo/c=cancel) ";
      ans=getchar();
      
      if (ans=='y') {
	cerr << "Value changed." << endl;
	display->dc.ofs16=ofstmp;
      } else 
	cerr << "Aborted." << endl;	  
    }
	
    if (ans=='c') break;
  }
}


/*******************************************************************/
inline bool
TSVGA_Analyzer::ProcessKeys()
{
  static int key;

  if ((key=vga_getkey())>0) {

    display->DisableClipping();

    int rv=display->ProcessControl(key);    
    
    switch (key) {

    case K_RESET:
      display->OnReset();
      break;

    case K_HELP:
      {
	extern char* gMainHelp;
	cerr << gMainHelp << endl;
	cerr << "Press a key..." << endl;
       	getchar();
	display->Show();
	display->Start();
      }
      break;

    case K_CALIBRATE: // calibrate
      display->Hide();      
      AutoCalibrate();
      cerr << "Press enter to return to display..." << endl;
      getchar();
      display->Show();      
      display->Start();
      break;

    case K_SAVEOPTIONS:
      char fname[256];
      display->Hide();
      cerr << endl << "Save current settings to file: ";
      cin >> fname;
      if (*fname)
	WriteOptions(fname); 
      display->Show();
      display->Start();
      break;

    case K_REVERTDEFAULTS:
      break;
            
    case K_AVGZERO:
      avg8=display->dc.ofs8; avg16=display->dc.ofs16;
      break;

    case K_EXIT:
      done=true;
      break;	             

    case K_METHOD_NEXT:
      CurrentMethod((CurrentMethod()%METHOD_COUNT)+1);
      break;

    case K_METHOD_PREV:
      {
	int n=CurrentMethod()-1;
	CurrentMethod((n<1)?METHOD_COUNT:--n);
      }
      break;
    }

    // dispatch return messages
    if (rv & UpdateSignal) OnUpdateSignalParameters();
    display->Refresh(rv);
    
    signal.flush();

    display->EnableClipping();    
  }

  return true;
}


/*******************************************************************/
void 
TSVGA_Analyzer::WriteOptions(const char *fname) const
{
  globalConfig gconfig_current=gconfig;
  OnRetrieveConfig(gconfig_current);
  cout << endl;
  gConfigDiff(gconfig_default, gconfig_current, cout);
  cout << endl << "Generate script with above args as file \"" 
       << fname << "\"? (y/n) ";
  string s;
  cin >> s;
  if (s[0]=='y' || s[0]=='Y') {
    cout << "Generating... ";
    ofstream out(fname);
    out << "#!/bin/sh" << endl;
    out << "svgafft ";
    gConfigDiff(gconfig_default, gconfig_current, out);
    out.close();
    chmod(fname, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
    cout << "done." << endl;
  } else
    cout << " -- canceled." << endl;
  sleep(1);
}


/*******************************************************************/
int
TSVGA_Analyzer::WindowSize() const
{
  assert(display);
  return display->WindowSize();
}


/*******************************************************************/
int
TSVGA_Analyzer::WindowSize(int w)
{
  int old=windowSize;

  assert(display);

  if (w>0) {
    display->WindowSize(w);
    buffer8.length(w);
    avg8.length(w);
    buffer16.length(w);
    avg16.length(w);
    windowSize=w;
  }

  return old;
}


/*******************************************************************/
void
TSVGA_Analyzer::OnSetConfig(const globalConfig& gc)
{
  DEBUG("TSVGA_Analzyer::OnSetConfig");
  assert(display);
  display->OnSetConfig(gc);
  display->Bands(gc.NumBands.val);
  WindowSize(gc.SampleSize.val);
}


/*******************************************************************/
void
TSVGA_Analyzer::OnRetrieveConfig(globalConfig& gc) const
{
  DEBUG("TSVGA_Analzyer::OnRetrieveConfig");
  assert(display);
  gc.NumBands.val=display->Bands();
  gc.SampleSize.val=WindowSize();
  display->OnRetrieveConfig(gc);  
  gc.Method.val=displayType;
}


/*******************************************************************/
void 
TSVGA_Analyzer::Initiate()
{
  vga_init();
  
  CurrentMethod(gconfig.Method.val);

  if (signal.open()) {

    assert(display);

    display->EnableClipping();

    done=false;  
    while (!done) {

      if (display->dc.bits8.isOn) {
	
	signal >> buffer8;

	Convert_Buffer_8XOR(buffer8, buffer8.length()); // convert to signed char

	if (display->dc.averaging.isOn) {
	  AverageWith(buffer8, avg8, display->dc.avgamt);
	  Type_Convert_Buffer(avg8, display->fft.data(), display->dc.ofs8);
	}	  
	else 
	  Type_Convert_Buffer(buffer8, display->fft.data(), display->dc.ofs8);
	
      } else {
	
	signal >> buffer16;

	if (display->dc.averaging.isOn) {
	  AverageWith(buffer16, avg16, display->dc.avgamt);
	  Type_Convert_Buffer(avg16, display->fft.data(), display->dc.ofs16);
	}	  
	else 
	  Type_Convert_Buffer(buffer16, display->fft.data(), display->dc.ofs16);

      } 
      
      display->fft.forward();
	
      display->OnUpdate();
      
      ProcessKeys();
    }
  }
  else 
    cerr << "error: could not open " << gconfig.Device.val << endl; 
  
  cerr << ENDMESSAGE << endl << endl;
}

/*******************************************************************/
