/**************************************************************************

    svga_display.h         
  
    Copyright (C) 1996 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: svga_display.h,v 0.5 1996/02/17 07:00:17 drewvel Exp $ */

#ifndef __SVGA_DISPLAY_H
#define __SVGA_DISPLAY_H

/*******************************************************************/
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include "../display.h"

#include "svga_color.h"

#define FWIDTH  8
#define FHEIGHT 8


/*******************************************************************/
struct TSVGA_DisplayInfo {
  TColor FG, BG, ScreenBG;
  TColor LineColor, AxesColor, LabelColor;
  TColor LowSegColor, MedSegColor, HiSegColor, PeakColor;
  
  unsigned long screenbg;
};


/*******************************************************************/
TSVGA_DisplayInfo& operator << (TSVGA_DisplayInfo&, const globalConfig&);
const TSVGA_DisplayInfo& operator >> (const TSVGA_DisplayInfo&, globalConfig&);


/*******************************************************************/
class TSVGA_Display:virtual public TDisplay {
  
friend class TSVGA_Analyzer;  
  
private:
  
  void init();
  
  void* fp; // SVGAlib font
  bool screen_on;
  int prevmode, mode;
  TSVGA_DisplayInfo sdi;
  int graph_x, graph_y; 
  int current_color; 

  void DrawTextP(int, int, const char *, int, int bg=-1);

  void OnPause();
  void CrossY(int, int, int, int);
  
protected:
  TSVGA_Analyzer* analyzer;

  // events
  void OnDisplayHelp();
  void OnCreate();
  void OnReset();
  bool OnStart();
  void OnStop();
  void OnUpdateControls();

  // drawing functions
  void ClearGraph();
  void DrawLine(int, int, int, int);
  void DrawLine(int, int, int, int, unsigned long);
  void DrawLines(XPoint*, int, int);
  void DrawText(int, int, const char *);
  void DrawCenteredText(int, int, const  char *);
  void FillRectangle(int, int, int, int);
  void FillRectangle(int, int, int, int, unsigned long);
  void SetColor(unsigned long);

  void EnableClipping();
  void DisableClipping();

  int ProcessDisplayControl(char);

  void OnSetDisplayConfig(const globalConfig&);
  void OnRetrieveDisplayConfig(globalConfig&);

public:
  TSVGA_Display(TSVGA_Analyzer*);  

  void Show();
  void Hide();
};



/*******************************************************************/
inline void
TSVGA_Display::SetColor(unsigned long c)
{
  current_color=c;
}


/*******************************************************************/
inline void
TSVGA_Display::DrawLine(int x1, int y1, int x2, int y2)
{
  gl_line(x1+graph_x, y1+graph_y, x2+graph_x, y2+graph_y, current_color);
}     


/*******************************************************************/
inline void
TSVGA_Display::DrawLine(int x1, int y1, int x2, int y2, unsigned long color)
{
  gl_line(x1+graph_x, y1+graph_y, x2+graph_x, y2+graph_y, color);
}     


/*******************************************************************/
inline void
TSVGA_Display::DrawLines(XPoint* points, int num, int cm)
{
  assert(num>0);
  if (cm==CoordModeOrigin) {
    --num;
    for (register int i=0; i<num; i++) {
#if 0
      cout << "(" << points[i].x+graph_x << "," << points[i].y+graph_y << ") "
	<< points[i+1].x+graph_x << "," << points[i+1].y+graph_y << ") " << endl;
#endif
      gl_line(points[i].x+graph_x, points[i].y+graph_y, 
	      points[i+1].x+graph_x, points[i+1].y+graph_y, 
	      current_color);
    }
  } 
  else {
    int px=points[0].x+graph_x, py=points[0].y+graph_y, x, y;
    for (register int i=1; i<num; i++) {
      x=px+points[i].x; y=px+points[i].y;
      gl_line(px, py, x, y, current_color); 
      px=x; py=y;
    }
  }    
}     


/*******************************************************************/
inline void
TSVGA_Display::DrawText(int x, int y, const char* str)
{
  assert(fp);
  gl_colorfont(FWIDTH, FHEIGHT, current_color, fp);
#ifdef SVGALIB_CHAR_CONST_CAST
  char* nc_str=const_cast<char*>(str);
  gl_write(x+graph_x, y+graph_y, nc_str);
#else
  gl_write(x+graph_x, y+graph_y, str);
#endif
}     


/*******************************************************************/
inline void
TSVGA_Display::DrawCenteredText(int x, int y, const char* str)
{
  assert(fp);
  int x_offset=-strlen(str)*FWIDTH/2;
  gl_colorfont(FWIDTH, FHEIGHT, current_color, fp);
#ifdef SVGALIB_CHAR_CONST_CAST
  char* nc_str=const_cast<char*>(str);
  gl_write(x+graph_x+x_offset, y+graph_y, nc_str);
#else
  gl_write(x+graph_x+x_offset, y+graph_y, str);
#endif
}     


/*******************************************************************/
inline void
TSVGA_Display::FillRectangle(int x, int y, int w, int h)
{
  gl_fillbox(x+graph_x, y+graph_y, w, h, current_color);
}     


/*******************************************************************/
inline void
TSVGA_Display::FillRectangle(int x, int y, int w, int h, unsigned long c)
{
  gl_fillbox(x+graph_x, y+graph_y, w, h, c);
}     


/*******************************************************************/
inline void
TSVGA_Display::EnableClipping()
{
  if (!__clip) {
    gl_enableclipping();
    gl_setclippingwindow(graph_x, graph_y, graph_x+di.width-1, graph_y+di.width);
  }
}     


/*******************************************************************/
inline void
TSVGA_Display::DisableClipping()
{
  if (__clip)
    gl_disableclipping();
}     


/*******************************************************************/
#endif /* __SVGA_DISPLAY_H */
