/**************************************************************************/
/* fftclass.h                                                             */
/**************************************************************************/
/**************************************************************************

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: fftclass.h,v 0.4 1996/02/08 05:57:56 drewvel Exp $ */

#ifndef __FFTCLASS_H
#define __FFTCLASS_H

/*******************************************************************/
#include <math.h>

#include "fft/ffts.h"
#include "buffer.h"


/*******************************************************************/
class TFFT {
private:
  integer l, l2;
  real N;
  TBuffer<real> databuf, work;
  
public:
  TFFT(int len=0);

  int length() const { return databuf.length(); }
  int length(int);

  TBuffer<real>& data() { return databuf; }
  TBuffer<real>& forward();
};


/*******************************************************************/
inline int
TFFT::length(int len)
{
  int old=l;

  if (len>0) {
    l=len;
    l2=l/2;
    N=(real) l2;
    old=databuf.length(l);     
    work.length(WORKDIMSIZE(l));
    INITFFT(&l, work);
  }

  return old; 
}


/*******************************************************************/
inline
TFFT::TFFT(int len)
  :l(len)
{ 
  length(l); 
}


/*******************************************************************/
inline TBuffer<real>&
TFFT::forward()
{
  assert(l>0);
  FORWARDFFT(&l, databuf, work);
  for (register int i=0, i2=0; i<l2; i++, i2+=2)
    databuf[i]=sqrt(databuf[i2]*databuf[i2]+databuf[i2+1]*databuf[i2+1])/N;
  return databuf;
}


/*******************************************************************/
#endif /* __FFTCLASS_H */
