/***************************************************************************

    index.h

    Copyright (C) 1995-96 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: index.h,v 0.3 1996/02/17 06:55:33 drewvel Exp $ */

#ifndef __INDEX_H
#define __INDEX_H

/*******************************************************************/
#include "buffer.h"
#include <math.h>

/*******************************************************************/
template <class T>
class TIndex {

private:
  int l;
  TBuffer<T> idx;
  
public:

  TIndex(int len=0) { length(l=len); }

  int length() const { return idx.length(); }
  int length(int);

  TBuffer<T>& index() { return idx; }

  void scale(T, T, int log=0);
};


/*******************************************************************/
template <class T>
inline int
TIndex<T>::length(int len)
{
  int old=l;

  if (len>0) {
    l=len;
    old=idx.length(l);     
  }

  return old; 
}


/*******************************************************************/
template <class T>
void
TIndex<T>::scale(T low, T high, int log)
{ 
  if (!log) {
    float m=(float) (((float) high-(float) low)/(float) length());
    float b=(float) low;

    for (int i=0; i<length(); i++) 
      index()[i]=(int) (m*(float) i+b);    
  } else {
    float l2=pow((float) length(),2);
    float a=(float) (((float) high-(float) low)/l2);
    float b=(float) ((float) low * l2)/l2;

    for (int i=0; i<length(); i++) 
      index()[i]=(int) (a*pow((float) i,2)+b);
  }
}


/*******************************************************************/
#endif /* __INDEX_H */
