/**************************************************************************

    method.C

    Copyright (C) 1996 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: method.C,v 0.6 1996/02/17 06:59:39 drewvel Exp $ */

/*******************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "method.h"
#include "common.h"
#include "gconfig.h"
#include "keymap.h"
#include "defvals.h"


/*******************************************************************/
TMethodControls::TMethodControls(const TMethod& Method)
  :method(Method), attenuation(1)
{
  vposition.name("v. position");
  vposition.val=0;
  vposition.step=1;

  LogY.Set();  
}


/*******************************************************************/
TMethodControls&
operator << (TMethodControls& mc, const globalConfig& gc)
{
  mc.attenuation=gc.Attenuation.val;

  mc.vposition=gc.VPos.val; 
  
  if (gc.LogScale.val) 
    mc.LogY.Set();
  else 
    mc.LogY.Clear();

  return mc;
}


/*******************************************************************/
const TMethodControls&
operator >> (const TMethodControls& mc, globalConfig& gc)
{
  gc.Attenuation.val=mc.attenuation;
  gc.VPos.val=mc.vposition.val;
  gc.LogScale.val=mc.LogY.isOn;

  return mc;
}



/*******************************************************************/
TMethod::TMethod()
  :mc(*this)
{
  DEBUG("TMethod constructing");
}


/*******************************************************************/
void
TMethod::MethodStatus(vector<string>& ms) const
{
  TDisplay::MethodStatus(ms);
  
  char s[128], q[32];

  sprintf(q, "[amp:%s]", mc.LogY.isOn ? "dB" : "lin");
  sprintf(s, "[atten:%-.2f] [vpos:%d] %-10s",
	  mc.attenuation,
	  mc.vposition.val,
	  q);  
  ms.push_back(s);
}


/*******************************************************************/
void
TMethod::OnUpdate()
{
  DEBUG("TMethod OnUpdate (random lines)");
  SetColor(di.fg);
  DrawLine(rand()%di.width, rand()%di.height, rand()%di.width, rand()%di.height);
}


/*******************************************************************/
int
TMethod::ProcessMethodControl(char control)
{
  DEBUG("Processing TMethod::MethodControls : " << control);

  int rv=0;

  switch (control) {

  case K_ATTEN_INC:
    mc.attenuation+=D_REAL_ATTENINC;
    rv|=UpdateControls;
    break;

  case K_ATTEN_DEC:
    mc.attenuation-=D_REAL_ATTENINC;
    if (mc.attenuation<D_REAL_ATTENINC) 
      mc.attenuation=D_REAL_ATTENINC;
    rv|=UpdateControls;
    break;

  case K_VPOS_INC:
    ++mc.vposition;
    rv|=UpdateControls;
    break;

  case K_VPOS_DEC:
    --mc.vposition;
    rv|=UpdateControls;
    break;

  case K_LOGYSCALE_TOGGLE:
    mc.LogY.Toggle();
    rv|=UpdateControls|UpdateAxes;
    break;
  }

  return rv;
}                                                            


/*******************************************************************/
void
TMethod::OnSetMethodConfig(const globalConfig& gc)
{
  TDisplay::OnSetMethodConfig(gc);

  DEBUG("TMethod OnSetMethodConfig");

  mc << gc;
}


/*******************************************************************/
void
TMethod::OnRetrieveMethodConfig(globalConfig& gc)
{
  TDisplay::OnRetrieveMethodConfig(gc);

  DEBUG("TMethod OnRetreiveMethodConfig");

  mc >> gc;
}


/*******************************************************************/
