/**************************************************************************

    method_2d.C

    Copyright (C) 1996 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: method_2d.C,v 0.3 1996/02/17 06:59:39 drewvel Exp $ */

/*******************************************************************/
#include <stdio.h>

#include "method_2d.h"


/*******************************************************************/
TMethod_2D::TMethod_2D()
{
  DEBUG("TMethod_2D constructing width==" << di.width);
}


/*******************************************************************/
void
TMethod_2D::OnUpdateAxes()
{
  DEBUG("Method: updating axes");

  // add label on major tick mark
  char label[64];

  BandScale(); // ensure frequencies are scaled correctly

  // draw axes
  int x, y_top=y_offset+1, y_height=di.height-y_offset, divisor;

  SetColor(di.axescolor);
  DrawLine(x_left_offset-1, 0, x_left_offset-1, y_offset);
  DrawLine(di.width-x_right_offset+1, 0, di.width-x_right_offset+1, y_offset);
  DrawLine(x_left_offset, y_offset+1, di.width-x_right_offset, y_offset+1);
  
  FillRectangle(0, y_offset+2, di.width, y_height, di.bg);    

  if (band_base.length()<50) divisor=4;
  else if (band_base.length()<100) divisor=8;
  else if (band_base.length()<200) divisor=16;
  else if (band_base.length()<300) divisor=24;
  else if (band_base.length()<400) divisor=32;
  else if (band_base.length()<500) divisor=40;
  else if (band_base.length()<600) divisor=48;
  else if (band_base.length()<700) divisor=56;
  else if (band_base.length()<800) divisor=64;
  else if (band_base.length()<900) divisor=72;
  else if (band_base.length()<1000) divisor=72;
  else if (band_base.length()<1100) divisor=80;
  else if (band_base.length()<1200) divisor=88;
  else divisor=96;

  // draw frequency axis
  for (int i=0; i<band_base.length(); i++) {
    
    x=(int) ((float) i*interval)+x_left_offset;

    int mi=/* some user control * */ (divisor/4+1)+i;

    if (!(mi%divisor)) {

      DrawLine(x, y_top, x, y_top+10);

      int freq=frequencies.index()[band_base.index()[i]];
      
      if (freq<1000) 
	sprintf(label, "%.0f", (float) freq);
      else
	sprintf(label, "%.2f", (float) freq/1000.0);
      
      SetColor(di.labelcolor);
      DrawCenteredText(x, y_top+12, label);
      SetColor(di.axescolor);
    }
    else if (!(mi%(divisor/2)))
      DrawLine(x, y_top, x, y_top+6);
    else if (!(mi%(divisor/4)))
      DrawLine(x, y_top, x, y_top+3);
    else if (divisor>=8)
      if (!(mi%(divisor/8)))
	DrawLine(x, y_top, x, y_top+2);
      else if (divisor>=16)
	if (!(mi%(divisor/16)))
	  DrawLine(x, y_top, x, y_top+1);
  }

  sprintf(label, "Hz/kHz [%s]", dc.LogX.isOn ? "log" : "linear");
  SetColor(di.labelcolor);
  DrawCenteredText((di.width+x_left_offset)/2, y_top+24, label);

  // draw amplitude axis  
  FillRectangle(0, 0, x_left_offset-1, y_offset, di.bg);    
  FillRectangle(di.width-x_right_offset+2, 0, x_right_offset, y_offset, di.bg);    

  int N=20;
  real intv=graph_height_scale/N;
  int w=3;

  SetColor(di.axescolor);  
  for (int k=0; k<N; k++) {
    int xl=x_left_offset-2;
    int xr=di.width-x_right_offset+2;
    int y=y_offset-(int) (intv*k);
    DEBUG("y: " << y);
    
    DrawLine(xl-w, y, xl, y);
    DrawLine(xr, y, xr+w, y);    

    sprintf(label, "%d", Amplitude(y));
    SetColor(di.labelcolor);
    DrawText(x_left_offset-strlen(label)*8-w-2, y-2, label);
    SetColor(di.axescolor);    
  }  

  SetColor(di.labelcolor);
  if (mc.LogY.isOn) {
    strcpy(label, "dB");
    DrawText(x_left_offset-(strlen(label)+2)*8-w-4, y_offset/2, label);
  }
}


/*******************************************************************/
int
TMethod_2D::Amplitude(int y) const
{
  real max, bits;  
  bits=dc.bits8.isOn ? 8.0 : 16.0;
  max=pow(2.0,bits);  
  if (mc.LogY.isOn) max=20*log10(max);
  return (int)
    ((y_offset-y)/graph_height_scale*max);
}


/*******************************************************************/
void
TMethod_2D::OnResizeMethod()
{
  DEBUG("TMethod_2D OnResizeMethod");
  DEBUG(" -- resize to " << di.width << "x" << di.height);

  const int x_left_axes_offset=48, x_right_axes_offset=8, y_axes_offset=40;

  x_left_offset=x_left_axes_offset;
  x_right_offset=x_right_axes_offset;
  y_offset=di.height-y_axes_offset;
  graph_height_scale=(real) y_offset;

  DEBUG("di.height: " << di.height << " ghs: " << graph_height_scale);

  assert(band_base.length()>0);

  interval = (float) GraphWidth() / (float) (band_base.length()-1);

  DEBUG("interval is " << interval);
}


/*******************************************************************/
