/**************************************************************************

    method_2d.h             

    Copyright (C) 1996 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: method_2d.h,v 0.4 1996/02/17 06:55:33 drewvel Exp $ */

#ifndef __METHOD_2D_H
#define __METHOD_2D_H

/*******************************************************************/
#include "method.h"
#include "buffer.h"
#include "rms.h"
#include "signal.h"


/*******************************************************************/
class TMethod_2D:public TMethod {

  friend class TAnalyzer;

protected:  
  int x_left_offset, x_right_offset, y_offset;
  real interval, graph_height_scale;

  void Y_Position(int, int&) const;

  int GraphWidth() const { return di.width-x_left_offset-x_right_offset; }
  int GraphHeight() const { return y_offset; }

  int Amplitude(int) const;

  void OnResizeMethod();
  void OnUpdateAxes();

public:
  TMethod_2D();
};


/*******************************************************************/
#define LinearAmpScale(data,max) ((data)/(max)*graph_height_scale)
#define DecibelAmpScale(data,max) ((20.0/(max))*log10((data)+1)*graph_height_scale)
inline void
TMethod_2D::Y_Position(int i, int& ny) const
{
  real data=rrms(fft.data(), band_base.index()[i], band_length.index()[i]);

  assert(data>=0.0);

  if (dc.bits8.isOn)
    if (mc.LogY.isOn) 
      data=DecibelAmpScale(data, TSignal::maxdb8);
    else
      data=LinearAmpScale(data, TSignal::maxval8);
  else
    if (mc.LogY.isOn) 
      data=DecibelAmpScale(data, TSignal::maxdb16);
    else
      data=LinearAmpScale(data, TSignal::maxval16);
     
  ny=(int) ( data / mc.attenuation ) + mc.vposition.val ;
  
  if (ny<0) ny=0;
}


/*******************************************************************/
#endif /* __METHOD_2D_H */
