/**************************************************************************

    bars_method.C

    Copyright (C) 1996 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: bars_method.C,v 1.1 1996/02/08 05:57:28 drewvel Exp $ */

/*******************************************************************/
#include <stdlib.h>
#include <math.h>

#include "../common.h"
#include "../gconfig.h"
#include "bars_method.h"


/*******************************************************************/
TBars_MethodControls::TBars_MethodControls(const TBars_Method& Method)
  :method(Method)
{
  holdCountMax=50;
  peaks.Set();
  falling.Set();
}


/*******************************************************************/
TBars_MethodControls&
operator << (TBars_MethodControls& bmc, const globalConfig& /* gc */)
{

  return bmc;
}


/*******************************************************************/
const TBars_MethodControls&
operator >> (const TBars_MethodControls& bmc, globalConfig& /* gc */)
{
  
  return bmc;
}


/*******************************************************************/
TBars_Method::TBars_Method()
  :bmc(*this)
{
  DEBUG("TBars_Method constructing width==" << di.width);
  segHeight=4;
  segSpace=4; 
  segDivider=8;
  segWidth=4;  
  medSegVal=hiSegVal=0;
  medSegPerc=70; hiSegPerc=90;
}


/*******************************************************************/
void
TBars_Method::OnResizeMethod()
{
  TMethod_2D::OnResizeMethod();

  DEBUG("Bars_Method OnResizeMethod");
  
  int num=band_base.length()/4;
  
  if (num<1) num=1;

  elem.length(num);

  bar_x_left_offset=x_left_offset+segWidth/2;

  bar_interval = (float) (di.width-(bar_x_left_offset+x_right_offset)) 
    / ((float) band_base.length() / (float) 4);  

  bar_y_offset=y_offset-segHeight;

  medSegVal=bar_y_offset-ThisPercentOf(medSegPerc, di.height);
  hiSegVal=bar_y_offset-ThisPercentOf(hiSegPerc, di.height);
}


/*******************************************************************/
bool
TBars_Method::OnStartMethod()
{
  holdCount=bmc.holdCountMax.val;
  
  int i;
  for (i=0; i<elem.length(); i++) {    
    elem[i].prevValue=elem[i].curValue=elem[i].peakValue=elem[i].y=bar_y_offset;
    elem[i].x=bar_x_left_offset+(int)((float)i*bar_interval);
  }

  medSegVal=bar_y_offset-ThisPercentOf(medSegPerc, di.height);
  hiSegVal=bar_y_offset-ThisPercentOf(hiSegPerc, di.height);

  return true;
}


/*******************************************************************/
void
TBars_Method::OnSetMethodConfig(const globalConfig& gc)
{
  TMethod_2D::OnSetMethodConfig(gc);

  DEBUG("TBars_Method OnSetMethodConfig");

  bmc << gc;

  medSegPerc=gc.MedSegVal.val;
  hiSegPerc=gc.HiSegVal.val;
  segHeight=gc.SegHeight.val;
  segWidth=gc.SegWidth.val;
  segSpace=gc.SegSpace.val;
  segDivider=segHeight+segSpace;
}


/*******************************************************************/
void
TBars_Method::OnRetrieveMethodConfig(globalConfig& gc)
{
  TMethod_2D::OnRetrieveMethodConfig(gc);

  DEBUG("TBars_Method OnRetreiveMethodConfig");

  bmc >> gc;

  gc.MedSegVal.val=medSegPerc;
  gc.HiSegVal.val=hiSegPerc;
  gc.SegHeight.val=segHeight;
  gc.SegWidth.val=segWidth;
  gc.SegSpace.val=segSpace;  
}


/*******************************************************************/
