/**************************************************************************

    bars_method.h

    Copyright (C) 1996 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: bars_method.h,v 1.1 1996/02/08 05:57:28 drewvel Exp $ */

#ifndef __BARS_METHOD_H
#define __BARS_METHOD_H

/*******************************************************************/
#include "../method_2d.h"
#include "../buffer.h"


class TBars_Method;

/*******************************************************************/
class TBars_MethodControls {
private:
  const TBars_Method& method;

public:

  TBars_MethodControls(const TBars_Method&);

  IntControl holdCountMax;
  BoolControl peaks;
  BoolControl falling;
};


/*******************************************************************/
TBars_MethodControls& operator << (TBars_MethodControls&, const globalConfig&);
const TBars_MethodControls& operator >> (const TBars_MethodControls&, globalConfig&);


/*******************************************************************/
class TBars_Method:public TMethod_2D {

  friend class TBars_MethodControls;

protected:
  int medSegPerc, hiSegPerc, clipPerc;
  int medSegVal, hiSegVal, clipVal;
  int segHeight, segSpace, segDivider, segWidth;

  struct Elem {
    int x, y;
    int prevValue, curValue, peakValue;
  };

  TBuffer<Elem> elem;
  int holdCount;
  int bar_y_offset, bar_x_left_offset;
  float bar_interval;

  TBars_MethodControls bmc;

  // events
  void OnResizeMethod();
  bool OnStartMethod();

  // global configuration accessors
  void OnSetMethodConfig(const globalConfig&);
  void OnRetrieveMethodConfig(globalConfig&);

public:
  TBars_Method();
};


/*******************************************************************/
#endif /* __BARS_METHOD_H */
