/**************************************************************************

    segs_method.C

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: segs_method.C,v 0.6 1996/02/17 07:00:36 drewvel Exp $ */

/*******************************************************************/
#include <stdlib.h>
#include <math.h>

#include "../common.h"
#include "../gconfig.h"
#include "segs_method.h"


/*******************************************************************/
TSegs_MethodControls::TSegs_MethodControls(const TSegs_Method& Method)
  :method(Method)
{
  divider=4;
  segHeight=3;
  segWidth=3;
  holdCountMax=50;
  peaks.Set();
  falling.Set();
}


/*******************************************************************/
TSegs_MethodControls&
operator << (TSegs_MethodControls& smc, const globalConfig& gc)
{
  smc.segHeight.val=gc.SegHeight.val;
  smc.segWidth.val=gc.SegWidth.val;

  return smc;
}


/*******************************************************************/
const TSegs_MethodControls&
operator >> (const TSegs_MethodControls& smc, globalConfig& gc)
{
  gc.SegHeight.val=smc.segHeight.val;
  gc.SegWidth.val=smc.segWidth.val;
  
  return smc;
}


/*******************************************************************/
TSegs_Method::TSegs_Method()
  :smc(*this)
{
  DEBUG("TSegs_Method constructing width==" << di.width);
}


/*******************************************************************/
void
TSegs_Method::OnUpdate()
{
  float intv=(float) bar_x_left_offset;
  int ny, nx, svy, num=bar.length();

  if (smc.peaks.isOn && !smc.falling.isOn) --holdCount;  

  int j=0;
  for (register int i=0; i<num; i++, j += smc.divider) {

    Y_Position(j, ny);

    ny/=smc.segHeight.val; ny*=smc.segHeight.val; // quantize

    ny=bar_y_offset-ny;

    nx=(int) intv; intv+=bar_interval;

    svy=bar[i].value;

    if (smc.peaks.isOn) {

      int& bp=bar[i].peak;
      
      if (smc.falling.isOn) {

	// let the peak "fall" by one notch
	bp+=smc.segHeight.val;      
	
	if (bp>bar_y_offset) bp=bar_y_offset;

	FillRectangle(nx, bp-smc.segHeight.val, smc.segWidth.val, smc.segHeight.val, di.bg);
	
	if (svy<bp) bp=svy;
	else 
	  FillRectangle(nx, bp, smc.segWidth.val, smc.segHeight.val, di.peakcolor);
      }
      else {

	if (bp>bar_y_offset) bp=bar_y_offset;
	
	if (svy<bp) {
	  FillRectangle(nx, bp, smc.segWidth.val, smc.segHeight.val, di.bg);
	  bp=svy;
	}
	else if (bp!=svy)
	  FillRectangle(nx, bp, smc.segWidth.val, smc.segHeight.val, di.peakcolor);

	if (!holdCount) {

	  holdCount=smc.holdCountMax.val;

	  int bx;
	  float bi=(float) bar_x_left_offset;
	  for (register int k=0; k<bar.length(); k++) {
	    bx=(int) bi; bi+=bar_interval;
	    FillRectangle(bx, bar[k].peak, smc.segWidth.val, smc.segHeight.val, di.bg);
	    bar[k].peak=bar_y_offset;
	  }
	}
      }
    }

    if (svy != ny) { // if it didn't change don't update
      
      // erase old segment
      FillRectangle(nx, svy, smc.segWidth.val, smc.segHeight.val, di.bg);

      // draw new segment
      FillRectangle(nx, ny, smc.segWidth.val, smc.segHeight.val, di.lowsegcolor);
      
      bar[i].value=ny;      
    }
  }
}


/*******************************************************************/
void
TSegs_Method::OnResizeMethod()
{
  TMethod_2D::OnResizeMethod();

  DEBUG("Segs_Method OnResizeMethod");
  
  int num=band_base.length()/smc.divider;
  
  if (num<1) num=1;

  bar.length(num);

  bar_x_left_offset=x_left_offset+smc.segWidth.val/2;

  bar_interval = (float) (di.width-(bar_x_left_offset+x_right_offset)) 
    / ((float) band_base.length() / (float) smc.divider);  

  bar_y_offset=y_offset-smc.segHeight.val;
}


/*******************************************************************/
bool
TSegs_Method::OnStartMethod()
{
  holdCount=smc.holdCountMax.val;
  
  int i;
  for (i=0; i<bar.length(); i++) 
    bar[i].value=bar[i].peak=bar_y_offset;

  return true;
}


/*******************************************************************/
void
TSegs_Method::OnSetMethodConfig(const globalConfig& gc)
{
  TMethod_2D::OnSetMethodConfig(gc);

  DEBUG("TSegs_Method OnSetMethodConfig");

  smc << gc;
}


/*******************************************************************/
void
TSegs_Method::OnRetrieveMethodConfig(globalConfig& gc)
{
  TMethod_2D::OnRetrieveMethodConfig(gc);

  DEBUG("TSegs_Method OnRetreiveMethodConfig");

  smc >> gc;
}


/*******************************************************************/
