/**************************************************************************

    svga_analyzer_methods.C

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: svga_analyzer_methods.C,v 1.4 1996/02/17 07:00:36 drewvel Exp $ */

/*******************************************************************/
#include <ostream.h>
#include <stdio.h>
#include <stdlib.h>

#include "../displays/svga_analyzer.h"
#include "../displays/svga_display.h"


/*******************************************************************/
#include "line_method.h"
class TSVGA_Line_Method:public TSVGA_Display, public TLine_Method {
public:
  TSVGA_Line_Method(TSVGA_Analyzer* analyzer):TSVGA_Display(analyzer) { }
};

#include "segs_method.h"
class TSVGA_Segs_Method:public TSVGA_Display, public TSegs_Method {
public:
  TSVGA_Segs_Method(TSVGA_Analyzer* analyzer):TSVGA_Display(analyzer) { }
};

#include "solid_bars_method.h"
class TSVGA_Solid_Bars_Method:public TSVGA_Display, public TSolid_Bars_Method {
public:
  TSVGA_Solid_Bars_Method(TSVGA_Analyzer* analyzer):TSVGA_Display(analyzer) { }
};

#include "seg_bars_method.h"
class TSVGA_Seg_Bars_Method:public TSVGA_Display, public TSeg_Bars_Method {
public:
  TSVGA_Seg_Bars_Method(TSVGA_Analyzer* analyzer):TSVGA_Display(analyzer) { }
};

#if 0
#include "test_method.h"
class TSVGA_Test_Method:public TSVGA_Display, public TTest_Method {
public:
  TSVGA_Test_Method(TSVGA_Analyzer* analyzer):TSVGA_Display(analyzer) { }
};
#endif

/*******************************************************************/
void 
TSVGA_Analyzer::NewDisplay(int type)
{
  DEBUG("TSVGA_Analyzer::NewDisplay of type " << type);
  
  DeleteDisplay();
  
  switch (type) {

  case LINE_METHOD:
    display=new TSVGA_Line_Method(this);    
    break;
    
  case SEGS_METHOD:
    display=new TSVGA_Segs_Method(this);    
    break;

  case SOLID_BARS_METHOD:
    display=new TSVGA_Solid_Bars_Method(this);
    break;
    
  case SEG_BARS_METHOD:
    display=new TSVGA_Seg_Bars_Method(this);
    break;
    
#if 0    
  case DISP_TEST_METHOD:
    display=new TSVGA_Test_Method(this);    
    break;
    
#endif
  default:
    char msg[128];
    sprintf(msg, "unknown display type %d", type);
    cerr << msg << endl;
    exit(1);
    break;
  }

  assert(display);
  
  displayType=type;
  
  if (display) {
    DEBUG("display changed, do update");

    OnSetConfig(gconfig);

    display->Initiate();
    
    display->Start();
  }
  else {
    cerr << "Couldn't allocate display, aborting..." << endl;
    exit(1);
  }  
}


/*******************************************************************/
