/**************************************************************************

    rms.h

    Copyright (C) 1995-96 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

// $Id: rms.h,v 1.6 1996/02/17 06:55:33 drewvel Exp $

#ifndef __RMS_H
#define __RMS_H

/*******************************************************************/
#include <math.h>
#include <assert.h>

#include "fft/ftypes.h"
#include "signal.h"
#include "buffer.h"

#define SQRT_2 1.41421356237

/*******************************************************************/
inline int 
irms(TBuffer<real>& buf, int start, int num)
{
  assert(num>0);
  real avg=0;
  num+=start;
  for (register int i=start; i<num; i++) avg+=buf[i]*buf[i];
  return (int) (sqrt(avg/(num-start))*(1/SQRT_2));
}


/*******************************************************************/
inline real 
rrms(TBuffer<real>& buf, int start, int num)
{
  assert(num>0);
  real avg=0;
  num+=start;
  for (register int i=start; i<num; i++) avg+=buf[i]*buf[i];
  return sqrt(avg/(num-start))*(1/SQRT_2);
}


/*******************************************************************/
inline int 
iavg(TBuffer<real>& buf, int start, int num)
{
  assert(num>0);
  real avg=0;
  num+=start;
  for (register int i=start; i<num; i++) avg+=buf[i];
  return (int) avg/(num-start);
}


/*******************************************************************/
inline real 
ravg(TBuffer<real>& buf, int start, int num)
{
  assert(num>0);
  real avg=0;
  num+=start;
  for (register int i=start; i<num; i++) avg+=buf[i];
  return avg/(num-start);
}


/*******************************************************************/
inline void
Convert_Buffer_8XOR(TSignal::sample8* b, int n)
{
  for (register int i=0; i<n; i++) b[i]^=0x80;
}


/*******************************************************************/
inline void
AverageWith(TBuffer<TSignal::sample8>& in, TBuffer<TSignal::sample8>& avg, real amt)
{
  for (register int i=0; i<avg.length(); i++) 
    avg[i]=(TSignal::sample8) (amt * (real) (in[i]-avg[i]) + (real) avg[i]);
}


/*******************************************************************/
inline void
AverageWith(TBuffer<TSignal::sample16>& in, TBuffer<TSignal::sample16>& avg, real amt)
{
  for (register int i=0; i<avg.length(); i++) 
    avg[i]=(TSignal::sample16) (amt * (real) (in[i]-avg[i]) + (real) avg[i]);
}


/*******************************************************************/
#endif /* __RMS_H */
