/**************************************************************************

    signal.h        

    Copyright (C) 1996 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: signal.h,v 0.13 1996/02/17 06:55:33 drewvel Exp $ */

#ifndef __SIGNAL_H
#define __SIGNAL_H

/*******************************************************************/
#include <string>

#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "buffer.h"


/*******************************************************************/
class TSignal {
public:
  typedef signed char sample8;
  typedef signed short sample16;
  const int numcalsamples=64;
  const float maxval8=255.0;
  const float maxval16=65535.0;
  const float maxdb8=48.1308036086;
  const float maxdb16=96.3296;

private:
  bool isOpen;
  int sig;

  void init();

protected:
  string filename;    

  // messangers
  virtual bool OnOpen();

public:
  TSignal();
  virtual ~TSignal();

  // interface
  const string& file() const { return filename; }
  bool open(const string&);
  bool open() { return isOpen; }  
  void close();
  virtual int ioctl(int, int parm=0) const { return parm; }
  virtual void flush() { } 

  virtual int bits() const { return 0; }
  virtual int bits(int) { return 0; }
  virtual int sample_rate() const { return 0; }
  virtual int sample_rate(int) { return 0; }
  virtual int channels() const { return 0; }
  virtual int channels(int) { return 0; }
  virtual int calibrate8(int) { return 0; }
  virtual int calibrate16(int) { return 0; }

  operator int () const { return sig; }
};


/*******************************************************************/
inline int
operator >> (TSignal& s, TBuffer<TSignal::sample8>& b)
{
  return read(s, b, b.size());
}


/*******************************************************************/
inline int
operator >> (TSignal& s, TBuffer<TSignal::sample16>& b)
{
  return read(s, b, b.size());
}

/*******************************************************************/
#endif /* __SIGNAL_H */
