/**************************************************************************

    vox_signal.C

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

/* $Id: vox_signal.C,v 0.2 1996/02/17 06:59:39 drewvel Exp $ */

/*******************************************************************/
#include <sys/soundcard.h>

#include "common.h"
#include "vox_signal.h"


/*******************************************************************/
void
TVox_Signal::init()
{
}


/*******************************************************************/
TVox_Signal::TVox_Signal()
{
  init();
  DEBUG("TVox_Signal constructed");
}


/*******************************************************************/
bool
TVox_Signal::OnOpen()
{
  ioctl(SNDCTL_DSP_SUBDIVIDE, 4);
  ioctl(SNDCTL_DSP_RESET);
  flush();  
  return true;
}


/*******************************************************************/
void
TVox_Signal::flush()
{
  ioctl(SNDCTL_DSP_SYNC);
}


/*******************************************************************/
int
TVox_Signal::bits() const
{
  return ioctl(SNDCTL_DSP_SETFMT, AFMT_QUERY);
}


/*******************************************************************/
int
TVox_Signal::bits(int b)
{
  flush();
  return ioctl(SNDCTL_DSP_SETFMT, b);
}


/*******************************************************************/
int
TVox_Signal::sample_rate() const
{
  return ioctl(SOUND_PCM_READ_RATE);
}


/*******************************************************************/
int
TVox_Signal::sample_rate(int r)
{
  flush();
  return ioctl(SOUND_PCM_WRITE_RATE, r);
}


/*******************************************************************/
int
TVox_Signal::channels() const
{
  return ioctl(SOUND_PCM_READ_CHANNELS);
}


/*******************************************************************/
int
TVox_Signal::channels(int c)
{
  flush();
  return ioctl(SOUND_PCM_WRITE_CHANNELS, c);
}


/*******************************************************************/
int
TVox_Signal::calibrate8(int n)
{
  int offset=0;
  sample8 c=0, delta=0, old=0;
  ioctl(SOUND_PCM_SYNC);
  for (int k=0; k<n; k++ ) {
    read(*this, &c, sizeof(sample8));
    c^=0x80;
    delta=c-old;
    offset+=delta;
    old=c;
  }
  return offset;
}


/*******************************************************************/
int
TVox_Signal::calibrate16(int n)
{
  int offset=0;
  sample16 c=0, delta=0, old=0;
  ioctl(SOUND_PCM_SYNC);
  for (int k=0; k<n; k++ ) {
    read(*this, &c, sizeof(sample16));
    delta=c-old;
    offset+=delta;
    old=c;
  }
  return offset;
}


/*******************************************************************/
